from django.conf import settings
from django.conf.urls import patterns, url

urlpatterns = patterns('campaign.views',
    url(r'^view/(?P<object_id>[\d]+)/$', 'view_online', {}, name="campaign_view_online"),
)

if getattr(settings, 'CAMPAIGN_SUBSCRIBE_CALLBACK', None):
    urlpatterns += patterns('campaign.views',
        url(r'^subscribe/$', 'subscribe', {}, name="campaign_subscribe"),
    )
    
if getattr(settings, 'CAMPAIGN_UNSUBSCRIBE_CALLBACK', None):
    urlpatterns += patterns('campaign.views',
        url(r'^unsubscribe/$', 'unsubscribe', {}, name="campaign_unsubscribe"),
    )


from django.conf import settings
from django.db import models
from django import forms
from django.utils import simplejson
 
class JSONWidget(forms.Textarea):
    def render(self, name, value, attrs=None):
        if not isinstance(value, basestring):
            value = simplejson.dumps(value, indent=2)
        return super(JSONWidget, self).render(name, value, attrs)
 
class JSONFormField(forms.CharField):
    def __init__(self, *args, **kwargs):
        kwargs['widget'] = JSONWidget
        super(JSONFormField, self).__init__(*args, **kwargs)
 
    def clean(self, value):
        if not value: return
        try:
            return simplejson.loads(value)
        except Exception, exc:
            raise forms.ValidationError(u'JSON decode error: %s' % (unicode(exc),))
 
class JSONField(models.TextField):
    __metaclass__ = models.SubfieldBase
 
    def formfield(self, **kwargs):
        return super(JSONField, self).formfield(form_class=JSONFormField, **kwargs)
 
    def to_python(self, value):
        if isinstance(value, basestring):
            value = simplejson.loads(value)
        return value
 
    def get_db_prep_save(self, value, connection=None):
        if value is None: return
        return simplejson.dumps(value)
 
    def value_to_string(self, obj):
        value = self._get_val_from_obj(obj)
        return self.get_db_prep_value(value)

if 'south' in settings.INSTALLED_APPS:
    from south.modelsinspector import add_introspection_rules
    add_introspection_rules([], ["^campaign\.fields\.JSONField"])        



from campaign.backends.base import BaseBackend

class SendMailBackend(BaseBackend):
    """simple backend which uses Django's built-in mail sending mechanisms"""
    
    def send_mail(self, email, fail_silently=False):
        """
        Parameters:
        
            ``email``: an instance of django.core.mail.EmailMessage
            ``fail_silently``: a boolean indicating if exceptions should 
            bubble up
            
        """
        return email.send(fail_silently=fail_silently)
        
backend = SendMailBackend()


# heavily based on Django's RequestContext
from django.conf import settings
from django.core.exceptions import ImproperlyConfigured
from django.template import Context

_mail_context_processors = None

# This is a function rather than module-level procedural code because we only
# want it to execute if somebody uses MailContext.
def get_mail_processors():
    global _mail_context_processors
    if _mail_context_processors is None:
        processors = []
        for path in getattr(settings, 'CAMPAIGN_CONTEXT_PROCESSORS', ()):
            i = path.rfind('.')
            module, attr = path[:i], path[i+1:]
            try:
                mod = __import__(module, {}, {}, [attr])
            except ImportError, e:
                raise ImproperlyConfigured('Error importing campaign processor module %s: "%s"' % (module, e))
            try:
                func = getattr(mod, attr)
            except AttributeError:
                raise ImproperlyConfigured('Module "%s" does not define a "%s" callable campaign processor' % (module, attr))
            processors.append(func)
        _mail_context_processors = tuple(processors)
    return _mail_context_processors
    
class MailContext(Context):
    """
    This subclass of template.Context automatically populates itself using
    the processors defined in CAMPAIGN_CONTEXT_PROCESSORS.
    Additional processors can be specified as a list of callables
    using the "processors" keyword argument.
    """
    def __init__(self, subscriber, dict=None, processors=None):
        Context.__init__(self, dict)
        if processors is None:
            processors = ()
        else:
            processors = tuple(processors)
        for processor in get_mail_processors() + processors:
            self.update(processor(subscriber))
        self.update({'recipient': subscriber})



# -*- coding: utf-8 -*-
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding model 'Newsletter'
        db.create_table('campaign_newsletter', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('name', self.gf('django.db.models.fields.CharField')(max_length=255)),
            ('description', self.gf('django.db.models.fields.TextField')(null=True, blank=True)),
        ))
        db.send_create_signal('campaign', ['Newsletter'])

        # Adding field 'Campaign.newsletter'
        db.add_column('campaign_campaign', 'newsletter',
                      self.gf('django.db.models.fields.related.ForeignKey')(to=orm['campaign.Newsletter'], null=True, blank=True),
                      keep_default=False)


    def backwards(self, orm):
        # Deleting model 'Newsletter'
        db.delete_table('campaign_newsletter')

        # Deleting field 'Campaign.newsletter'
        db.delete_column('campaign_campaign', 'newsletter_id')


    models = {
        'campaign.blacklistentry': {
            'Meta': {'ordering': "('-added',)", 'object_name': 'BlacklistEntry'},
            'added': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'email': ('django.db.models.fields.EmailField', [], {'max_length': '75'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'})
        },
        'campaign.bounceentry': {
            'Meta': {'ordering': "('email',)", 'object_name': 'BounceEntry'},
            'email': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'}),
            'exception': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'})
        },
        'campaign.campaign': {
            'Meta': {'ordering': "('name', 'sent')", 'object_name': 'Campaign'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'newsletter': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['campaign.Newsletter']", 'null': 'True', 'blank': 'True'}),
            'online': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'recipients': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['campaign.SubscriberList']", 'symmetrical': 'False'}),
            'sent': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'sent_at': ('django.db.models.fields.DateTimeField', [], {'null': 'True', 'blank': 'True'}),
            'template': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['campaign.MailTemplate']"})
        },
        'campaign.mailtemplate': {
            'Meta': {'ordering': "('name',)", 'object_name': 'MailTemplate'},
            'html': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'plain': ('django.db.models.fields.TextField', [], {}),
            'subject': ('django.db.models.fields.CharField', [], {'max_length': '255'})
        },
        'campaign.newsletter': {
            'Meta': {'ordering': "('name',)", 'object_name': 'Newsletter'},
            'description': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '255'})
        },
        'campaign.subscriberlist': {
            'Meta': {'ordering': "('name',)", 'object_name': 'SubscriberList'},
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['contenttypes.ContentType']"}),
            'email_field_name': ('django.db.models.fields.CharField', [], {'max_length': '64'}),
            'filter_condition': ('campaign.fields.JSONField', [], {'default': "'{}'"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '255'})
        },
        'contenttypes.contenttype': {
            'Meta': {'ordering': "('name',)", 'unique_together': "(('app_label', 'model'),)", 'object_name': 'ContentType', 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        }
    }

    complete_apps = ['campaign']


from django import template
from django.core.mail import EmailMultiAlternatives
from django.core.urlresolvers import reverse
from django.contrib.sites.models import Site
from campaign.context import MailContext

class BaseBackend(object):
    """base backend for all campaign backends"""
    
    def send_campaign(self, campaign, fail_silently=False):
        """
        Does the actual work
        """  
        from campaign.models import BlacklistEntry
          
        subject = campaign.template.subject
        text_template = template.Template(campaign.template.plain)
        if campaign.template.html is not None and campaign.template.html != u"":
            html_template = template.Template(campaign.template.html)
        
        sent = 0
        used_addresses = []
        for recipient_list in campaign.recipients.all():
            for recipient in recipient_list.object_list():
                # never send mail to blacklisted email addresses
                recipient_email = getattr(recipient, recipient_list.email_field_name)
                if not BlacklistEntry.objects.filter(email=recipient_email).count() and not recipient_email in used_addresses:
                    msg = EmailMultiAlternatives(subject, to=[recipient_email,])
                    context = MailContext(recipient)
                    if campaign.online:
                        context.update({'view_online_url': reverse("campaign_view_online", kwargs={'object_id': campaign.pk}),
                                        'site_url': Site.objects.get_current().domain,
                                        'recipient_email': recipient_email})
                    msg.body = text_template.render(context)
                    if campaign.template.html is not None and campaign.template.html != u"":
                        html_content = html_template.render(context)
                        msg.attach_alternative(html_content, 'text/html')
                    sent += self.send_mail(msg, fail_silently)
                    used_addresses.append(recipient_email)
        return sent
    
    def send_mail(self, email, fail_silently=False):
        raise NotImplementedError
        


from django.conf import settings
from mailer import send_mail
from campaign.backends.base import BaseBackend

class DjangoMailerBackend(BaseBackend):
    """simple backend using django-mailer to queue and send the mails"""
    
    def send_mail(self, email, fail_silently=False):
        subject = email.subject
        body = email.body
        # django_mailer does not support multi-part messages so we loose the
        # html content
        return send_mail(subject, body, settings.DEFAULT_FROM_EMAIL, email.recipients(), 
                    fail_silently=fail_silently)
                    
backend = DjangoMailerBackend()


from distutils.core import setup

setup(
    name='django-campaign',
    version=__import__('campaign').__version__,
    description='A basic newsletter app for the Django webframework',
    long_description=open('docs/index.txt').read(),
    author='Arne Brodowski',
    author_email='arne@rcs4u.de',
    license="BSD",
    url='https://github.com/arneb/django-campaign/',
    download_url='http://code.google.com/p/django-campaign/downloads/list',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Framework :: Django',
    ],
    packages = (
        'campaign',
        'campaign.backends',
        'campaign.migrations',
    ),
    package_data = {
        'campaign': 
            ['templates/admin/campaign/campaign/*.html',
             'templates/campaign/*.html'],
    }
)



from campaign.backends.base import BaseBackend

class DebugBackend(BaseBackend):
    """ an example backend which just prints out the email instead of sending it

        NOTE: This backend is obsolete since Django now has a built-in 
        mechanism to print emails to the console. Just use the send_mail
        Backend and configure Django accordingly.
        
    """
    
    def send_mail(self, email, fail_silently=False):
        print "Subject: %s" % email.subject
        print "To: %s" % email.recipients()
        print "======"
        print email.message().as_string() # the actual email message
        print "======"
        return 0
        
backend = DebugBackend()


from django.shortcuts import render_to_response
from django import template
from django import forms
from django.utils.functional import update_wrapper
from django.contrib import admin
from django.http import HttpResponseRedirect
from django.contrib.admin.util import unquote
from django.contrib import messages
from django.core.exceptions import PermissionDenied
from django.utils.translation import ugettext as _
from django.utils.encoding import force_unicode
from django.utils.safestring import mark_safe
from campaign.models import MailTemplate, Campaign, BlacklistEntry, \
BounceEntry, SubscriberList, Newsletter
 

class CampaignAdmin(admin.ModelAdmin):
    filter_horizontal=('recipients',)
    list_display=('name', 'newsletter', 'sent', 'sent_at', 'online')
    send_template = None
    
    def has_send_permission(self, request, obj):
        """
        Subclasses may override this and implement more granular permissions.
        TODO: integrate with django's permisson system
        """
        return request.user.is_superuser
        
        
    def send_view(self, request, object_id, extra_context=None):
        """
        Allows the admin to send out the mails for this campaign.
        """
        model = self.model
        opts = model._meta
        
        try:
            obj = model._default_manager.get(pk=unquote(object_id))
        except model.DoesNotExist:
            # Don't raise Http404 just yet, because we haven't checked
            # permissions yet. We don't want an unauthenticated user to be able
            # to determine whether a given object exists.
            obj = None
        
        if not self.has_send_permission(request, obj):
            raise PermissionDenied
        
        if obj is None:
            raise Http404(_('%(name)s object with primary key %(key)r does not exist.') % {'name': force_unicode(opts.verbose_name), 'key': escape(object_id)})
        
        if request.method == 'POST':
            if not request.POST.get('send', None) == '1':
                raise PermissionDenied
            
            num_sent = obj.send()
            messages.success(request, _(u'The %(name)s "%(obj)s" was successfully sent. %(num_sent)s messages delivered.' %  {'name': force_unicode(opts.verbose_name), 'obj': force_unicode(obj), 'num_sent': num_sent,}))
            return HttpResponseRedirect('../')

        
        def form_media():
            from django.conf import settings
            css = ['css/forms.css',]
            return forms.Media(css={'screen': ['%sadmin/%s' % (settings.STATIC_URL, url) for url in css]})
        
        media = self.media + form_media()

        context = {
            'title': _('Send %s') % force_unicode(opts.verbose_name),
            'object_id': object_id,
            'object': obj,
            'is_popup': request.REQUEST.has_key('_popup'),
            'media': mark_safe(media),
            'app_label': opts.app_label,
            'opts': opts,
        }
        context.update(extra_context or {})
        
        return render_to_response(self.send_template or
            ['admin/%s/%s/send_object.html' % (opts.app_label, opts.object_name.lower()),
            'admin/%s/send_object.html' % opts.app_label,
            'admin/send_object.html'], context, context_instance=template.RequestContext(request))
    
        
    def get_urls(self):
        from django.conf.urls.defaults import patterns, url

        def wrap(view):
            def wrapper(*args, **kwargs):
                return self.admin_site.admin_view(view)(*args, **kwargs)
            return update_wrapper(wrapper, view)

        info = self.admin_site.name, self.model._meta.app_label, self.model._meta.module_name
        
        super_urlpatterns = super(CampaignAdmin, self).get_urls()
        urlpatterns = patterns('',
            url(r'^(.+)/send/$',
                wrap(self.send_view),
                name='%sadmin_%s_%s_send' % info),
        )
        urlpatterns += super_urlpatterns
        
        return urlpatterns


admin.site.register(Campaign, CampaignAdmin)
admin.site.register(MailTemplate)
admin.site.register(BlacklistEntry)
admin.site.register(BounceEntry)
admin.site.register(SubscriberList)
admin.site.register(Newsletter)
